module Fuel
  module CLI
    class Main < Thor
      include Fuel::Util

      package_name "fuel"

      desc "setup", "Set up the CLI interactively"
      def setup
        say   "############################################\n\n", [:cyan, :bold]
        say flame, [:cyan, :bold]
        say "\n########### Welcome to Fuel-CLI! ###########\n\n", [:cyan, :bold]

        setup_gerrit
        setup_jira
        setup_jenkins

        say "You are good to go!", [:green, :bold]
        say 'You have "fuel", "gerrit", "jira" and "jenkins" executables in your PATH.', :green
        say 'Use the "help" command (e.g. "gerrit help") to get started.', :green
      end

      option :gerrit_url
      option :jira_url
      option :jenkins_url
      desc "config", "Set configuration values"
      def config
        raise Thor::Error, set_color("Provide at least one key-value pair!", :red) if options.empty?
        if options.has_key?('gerrit_url')
          Config.set('gerrit', 'url', options['gerrit_url'])
        end
        if options.has_key?('jira_url')
          Config.set('jira', 'url', options['jira_url'])
        end
        if options.has_key?('jenkins_url')
          Config.set('jenkins', :server_url, options['jenkins_url'])
        end
      end

      private

      def setup_gerrit
        say "[Setting up gerrit]", :yellow
        gerrit_url = ask("URL:").chomp('/')
        self.class.new.invoke :config, [], gerrit_url: gerrit_url
        gerrit_user = ask "User name:"
        say "Your Gerrit user settings will open now, please generate an HTTP password there."
        sleep 3
        `open #{gerrit_url}/#/settings/http-password`
        Gerrit.new.invoke :auth, [gerrit_user]

        say "gerrit configured successfully!\n\n", :green
      end

      def setup_jira
        say "[Setting up jira]", :yellow
        jira_url = ask "URL:"
        self.class.new.invoke :config, [], jira_url: jira_url
        jira_user = ask "User name:"
        Jira.new.invoke :auth, [jira_user]

        say "jira configured successfully!\n\n", :green
      end

      def setup_jenkins
        say "[Setting up jenkins]", :yellow
        jenkins_url = ask "URL:"
        self.class.new.invoke :config, [], jenkins_url: jenkins_url
        jenkins_user = ask "User name:"
        Jenkins.new.invoke :auth, [jenkins_user]

        say "jenkins configured successfully!\n\n", :green
      end

      def flame
        <<-EOT
                            `.--..`
         `-/+syyso/.      .sdddddddhs+:.
      -+ydddddddddddh/`  .dddh:.`.:+sdddho:`
    :yhsso+oshddddddddds/yddy`       `+ddddo`
   `.   ```   .+ddds/-:/+o+:     `-+ydds/.
     .:odddh+   `sdddho:`    .:+ydds/.
`:ohdddddddddy`   /ddddddhhhddhs/.
   `./oyhddhs:`    .ohhdhys+:`
        EOT
      end
    end
  end
end