require 'fileutils'
require 'yaml'

module Fuel
  module Util
    class Config
      CONFIG_PATH = File.expand_path("~/.fuel")
      CONFIG_FILENAME = "#{CONFIG_PATH}/cliconfig"

      def self.set(*keys, value)
        ensure_config_exists

        config = load
        key = keys.pop
        h = keys.inject(config) { |memo, k| memo[k] ||= {}; memo[k] }
        h[key] = value
        save(config)
      end

      def self.set64(*keys, value)
        self.set(*keys, Base64::encode64(value))
      end

      def self.get(*keys)
        ensure_config_exists

        config = load
        keys.inject(config, :fetch)
      rescue KeyError
        nil
      end

      def self.get64(*keys)
        value = self.get(*keys)
        value && Base64::decode64(value)
      end

      def self.add(key, value)
        ensure_config_exists

        config = load
        config[key] ||= []
        config[key] << value
        save(config)
      end

      private

      def self.load
        @config ||= (YAML.load_file(CONFIG_FILENAME) || {})
      end

      def self.save(config)
        File.open(CONFIG_FILENAME, "w") { |f| f.write(YAML.dump(config)) }
      end

      def self.ensure_config_exists
        FileUtils.mkdir_p(CONFIG_PATH)
        FileUtils.touch(CONFIG_FILENAME)
      end
    end
  end
end